@@ -1,9 +1,22 @@
-from .__core__ import PyotCore, PyotStatic
 from datetime import datetime, timedelta
-from typing import List, get_type_hints, Iterator
+from collections import defaultdict
+from typing import List, Iterator, Dict
 import asyncio
 
-from pyot.utils import fast_copy
+from pyot.utils import AutoData, dict_key_value_swap
+from pyot.core.functional import turbo_copy, handle_import_error
+from .__core__ import PyotCore, PyotStatic
+
+try:
+    import roleml
+except (ImportError, ValueError):
+    pass
+
+try:
+    import roleidentification
+    champion_roles = AutoData(lambda: defaultdict(lambda: {}, roleidentification.pull_data()))
+except ImportError:
+    pass
 
 
 # PYOT STATIC OBJECTS
@@ -37,7 +50,7 @@ class MatchParticipantStatData(PyotStatic):
     rune_ids: List[int]
     stat_rune_ids: List[int]
     rune_vars: List[List[int]]
-    rune_style: int
+    rune_main_style: int
     rune_sub_style: int
     kills: int
     deaths: int
@@ -98,7 +111,7 @@ class MatchParticipantStatData(PyotStatic):
 
     class Meta(PyotStatic.Meta):
         raws = ["item_ids", "dominion_scores", "rune_ids", "spell_ids", "rune_vars", "stat_rune_ids"]
-        renamed = {"time_c_cing_others": "time_ccing_others"}
+        renamed = {"time_c_cing_others": "time_ccing_others", "perk_primary_style": "rune_main_style", "perk_sub_style": "rune_sub_style"}
 
     @property
     def items(self) -> List["Item"]:
@@ -159,7 +172,8 @@ class MatchEventData(PyotStatic):
     team_id: int
     position: MatchPositionData
     killer_id: int
-    timestamp: timedelta
+    timestamp: float
+    time: timedelta
     assisting_participant_ids: List[int]
     building_type: str
     victim_id: int
@@ -167,10 +181,9 @@ class MatchEventData(PyotStatic):
     class Meta(PyotStatic.Meta):
         raws = ["assisting_participant_ids"]
 
-    def __getattribute__(self, name):
-        if name == "timestamp":
-            return timedelta(super().__getattribute__(name))
-        return super().__getattribute__(name)
+    @property
+    def time(self) -> timedelta:
+        return timedelta(milliseconds=self.timestamp)
 
     @property
     def after_item(self) -> "Item":
@@ -205,20 +218,20 @@ class MatchEventData(PyotStatic):
 
 class MatchParticipantTimelineData(PyotStatic):
     participant_id: int
-    creeps_per_min_deltas: List[float]
-    xp_per_min_deltas: List[float]
-    gold_per_min_deltas: List[float]
-    cs_diff_per_min_deltas: List[float]
-    xp_diff_per_min_deltas: List[float]
-    damage_taken_per_min_deltas: List[float]
-    damage_taken_diff_per_min_deltas: List[float]
+    creeps_per_min_deltas: Dict[str, float]
+    xp_per_min_deltas: Dict[str, float]
+    gold_per_min_deltas: Dict[str, float]
+    cs_diff_per_min_deltas: Dict[str, float]
+    xp_diff_per_min_deltas: Dict[str, float]
+    damage_taken_per_min_deltas: Dict[str, float]
+    damage_taken_diff_per_min_deltas: Dict[str, float]
     role: str
     lane: str
     frames: List[MatchFrameData]
     events: List[MatchEventData]
 
     class Meta(PyotStatic.Meta):
-        raws = ["creeps_per_min_deltas", "xp_per_min_deltas", "gold_per_min_deltas", "cs_diff_per_min_deltas", 
+        raws = ["creeps_per_min_deltas", "xp_per_min_deltas", "gold_per_min_deltas", "cs_diff_per_min_deltas",
             "xp_diff_per_min_deltas", "damage_taken_per_min_deltas", "damage_taken_diff_per_min_deltas"]
 
 
@@ -260,7 +273,7 @@ class MatchParticipantData(PyotStatic):
     @property
     def account(self) -> "Summoner":
         from .summoner import Summoner
-        return Summoner(account_id=self.account_id, platform=self.platform)
+        return Summoner(account_id=self.account_id, platform=self.current_platform)
 
     @property
     def current_account(self) -> "Summoner":
@@ -270,16 +283,16 @@ class MatchParticipantData(PyotStatic):
     @property
     def summoner(self) -> "Summoner":
         from .summoner import Summoner
-        return Summoner(id=self.summoner_id, platform=self.platform)
+        return Summoner(id=self.summoner_id, platform=self.current_platform)
 
     @property
     def profile_icon(self) -> "ProfileIcon":
         from .profileicon import ProfileIcon
-        return ProfileIcon(id=self.profile_icon_id, locale=self.to_locale(self.platform))
+        return ProfileIcon(id=self.profile_icon_id, locale=self.to_locale(self.current_platform))
 
 
 class MatchTeamData(PyotStatic):
-    team_id: int
+    id: int
     win: bool
     first_blood: bool
     first_tower: bool
@@ -297,6 +310,9 @@ class MatchTeamData(PyotStatic):
     bans: List[MatchBanData]
     participants: List[MatchParticipantData]
 
+    class Meta(PyotStatic.Meta):
+        renamed = {"team_id": "id"}
+
 
 class MatchHistoryData(PyotStatic):
     platform: str
@@ -304,18 +320,17 @@ class MatchHistoryData(PyotStatic):
     champion_id: int
     queue_id: int
     season_id: int
+    timestamp: int
     creation: datetime
     role: str
     lane: str
 
     class Meta(PyotStatic.Meta):
-        renamed = {"platform_id": "platform", "game_id": "id", "champion": "champion_id", "queue": "queue_id",
-            "season": "season_id", "timestamp": "creation"}
+        renamed = {"platform_id": "platform", "game_id": "id", "champion": "champion_id", "queue": "queue_id", "season": "season_id"}
 
-    def __getattribute__(self, name):
-        if name == "creation":
-            return datetime.fromtimestamp(super().__getattribute__(name)//1000)
-        return super().__getattribute__(name)
+    @property
+    def creation(self) -> datetime:
+        return datetime.fromtimestamp(self.timestamp//1000)
 
     @property
     def champion(self) -> "Champion":
@@ -331,51 +346,20 @@ class MatchHistoryData(PyotStatic):
     def match(self) -> "Match":
         return Match(id=self.id, platform=self.platform)
 
-    @property
-    def match_timeline(self) -> "MatchTimeline":
-        return MatchTimeline(id=self.id, platform=self.platform)
-
     @property
     def timeline(self) -> "Timeline":
         return Timeline(id=self.id, platform=self.platform)
 
 
 class MatchFrameMinuteData(PyotStatic):
-    data: List[MatchFrameData]
+    participant_frames: List[MatchFrameData]
+    events: List[MatchEventData]
+    timestamp: int
+    time: timedelta
 
-    def __init__(self, data):
-        data = {'data': data}
-        super().__init__(data)
-
-    def __getitem__(self, item):
-        if not isinstance(item, int):
-            return super().__getitem__(item)
-        return self.data[item]
-
-    def __iter__(self) -> Iterator[MatchFrameData]:
-        return iter(self.data)
-
-    def __len__(self):
-        return len(self.data)
-
-
-class MatchEventMinuteData(PyotStatic):
-    data: List[MatchEventData]
-
-    def __init__(self, data):
-        data = {'data': data}
-        super().__init__(data)
-
-    def __getitem__(self, item):
-        if not isinstance(item, int):
-            return super().__getitem__(item)
-        return self.data[item]
-
-    def __iter__(self) -> Iterator[MatchEventData]:
-        return iter(self.data)
-
-    def __len__(self):
-        return len(self.data)
+    @property
+    def time(self):
+        return timedelta(milliseconds=self.timestamp)
 
 
 # PYOT CORE OBJECTS
@@ -388,223 +372,202 @@ class Match(PyotCore):
     map_id: int
     season_id: int
     queue_id: int
+    creation_millis: int
+    duration_secs: int
     creation: datetime
-    duration: timedelta #not milliseconds
+    duration: timedelta
     platform: str
     teams: List[MatchTeamData]
-    blue_team: MatchTeamData
-    red_team: MatchTeamData
     tournament_code: str
+    include_timeline: bool
 
     class Meta(PyotCore.Meta):
+        raw_timeline: Dict
         rules = {
             "match_v4_tournament_match": ["tournament_code", "id"],
             "match_v4_match": ["id"],
         }
-        renamed = {"game_id": "id", "platform_id": "platform", "game_creation": "creation", "game_duration": "duration",
+        renamed = {"game_id": "id", "platform_id": "platform", "game_creation": "creation_millis", "game_duration": "duration_secs",
             "game_version": "version", "game_mode": "mode", "game_type": "type", "queue": "queue_id"}
 
-    def __getattribute__(self, name):
-        if name == "creation":
-            return datetime.fromtimestamp(super().__getattribute__(name)//1000)
-        elif name == "duration":
-            return timedelta(seconds=super().__getattribute__(name))
-        return super().__getattribute__(name)
-
-    def __init__(self, id: int = None, tournament_code: str = None, platform: str = None):
+    def __init__(self, id: int = None, tournament_code: str = None, include_timeline: bool = False, platform: str = None):
         self._lazy_set(locals())
 
+    async def get(self, sid: str = None, pipeline: str = None, deepcopy: bool = False):
+        if not self.include_timeline:
+            return await super().get(sid, pipeline, deepcopy)
+        get_timeline = asyncio.create_task(Timeline(id=self.id, platform=self.platform).get(sid, pipeline, deepcopy))
+        get_match = asyncio.create_task(super().get(sid, pipeline, deepcopy))
+        await get_match
+        timeline = await get_timeline
+        self._meta.raw_timeline = timeline.raw()
+        self._inject_timeline(timeline.dict())
+        return self
+
+    def _turbo_copy(self, data):
+        data = data.copy()
+        data['teams'] = turbo_copy(data['teams'], 2)
+        data['participants'] = turbo_copy(data['participants'], 2)
+        for i, p in enumerate(data['participants']):
+            data['participants'][i]['stats'] = p['stats'].copy()
+            data['participants'][i]['timeline'] = p['timeline'].copy()
+        return data
+
     def _transform(self, data):
+        data = self._turbo_copy(data)
         if data["teams"][0]["teamId"] == 100:
             blue_team = data["teams"][0]
             red_team = data["teams"][1]
-        elif data["teams"][0]["teamId"] == 200:
+        else:
             red_team = data["teams"][0]
             blue_team = data["teams"][1]
-        else:
-            blue_team = data["teams"][0]
-            red_team = data["teams"][1]
 
         blue_team["participants"] = []
         red_team["participants"] = []
 
-        if blue_team["win"] == "Win":
-            blue_team["win"] = True
-        else:
-            blue_team["win"] = False
-        
-        if red_team["win"] == "Win":
-            red_team["win"] = True
-        else:
-            red_team["win"] = False
+        for team in (blue_team, red_team):
+            if team["win"] == "Win": team["win"] = True
+            else: team["win"] = False
 
         for p in data["participants"]:
-            p["spellIds"] = [p.pop("spell1Id"), p.pop("spell2Id")]
             stats = p["stats"]
-            stats["spellIds"] = p["spellIds"]
-            
-            stats["dominionScores"] = []
-            for i in range(10):
-                score = stats.pop("playerScore"+str(i))
-                stats["dominionScores"].append(score)
-            
-            stats["itemIds"] = []
-            for i in range(7):
-                item = stats.pop("item"+str(i))
-                stats["itemIds"].append(item)
+            p["spellIds"] = stats["spellIds"] = [p.pop("spell1Id", None), p.pop("spell2Id", None)]
 
-            stats["runeIds"] = []
-            stats["runeVars"] = []
-            for i in range(6):
-                rune = stats.pop("perk"+str(i))
-                stats["runeIds"].append(rune)
-                statset = []
-                for j in range(1, 4):
-                    st = stats.pop("perk"+str(i)+"Var"+str(j))
-                    statset.append(st)
-                stats["runeVars"].append(statset)
-            
-            stats["runeStyle"] = stats.pop("perkPrimaryStyle", None)
-            stats["runeSubStyle"] = stats.pop("perkSubStyle", None)
+            stats["dominionScores"] = [stats.pop("playerScore"+str(i), None) for i in range(10)]
+            stats["itemIds"] = [stats.pop("item"+str(i), None) for i in range(7)]
+            stats["runeIds"] = [stats.pop("perk"+str(i), None) for i in range(6)]
+            stats["runeVars"] = [[stats.pop("perk"+str(i)+"Var"+str(j), None) for j in range(1, 4)] for i in range(6)]
+            stats["statRuneIds"] = [stats.pop("statPerk"+str(i), None) for i in range(3)]
 
-            stats["statRuneIds"] = []
-            for i in range(3):
-                statrune = stats.pop("statPerk"+str(i))
-                stats["statRuneIds"].append(statrune)
-
-            timeline = p["timeline"]
-
-            for key, val in timeline.items():
-                if "Deltas" in key:
-                    deltaset = []
-                    for i in range(6):
-                        try: deltaset.append(val[f"{i*10}-{i*10+10}"])
-                        except KeyError: break
-                    timeline[key] = deltaset
-            
-            if p["teamId"] == 100:
-                blue_team["participants"].append(p)
-            elif p["teamId"] == 200:
-                red_team["participants"].append(p)
+            if p["teamId"] == 100: blue_team["participants"].append(p)
+            elif p["teamId"] == 200: red_team["participants"].append(p)
 
         for pi in data["participantIdentities"]:
             try:
-                if pi["player"]["platformId"].lower() == "na":
-                    pi["player"]["platformId"] = "NA1"
-                if pi["player"]["currentPlatformId"].lower() == "na":
-                    pi["player"]["currentPlatformId"] = "NA1"
-                pid = pi["participantId"]
-                for p in blue_team["participants"]:
-                    if p["participantId"] == pid:
-                        p.update(pi["player"])
-                for p in red_team["participants"]:
-                    if p["participantId"] == pid:
-                        p.update(pi["player"])
-            except KeyError:
-                pass
-        
-        data.pop("participants")
-        data.pop("participantIdentities")
-        data["blueTeam"] = blue_team
-        data["redTeam"] = red_team
+                for platform_key in ("platformId", "currentPlatformId"):
+                    if pi["player"][platform_key].lower() == "na":
+                        pi["player"][platform_key] = "NA1"
+                for team in (blue_team, red_team):
+                    for p in team["participants"]:
+                        if p["participantId"] == pi["participantId"]:
+                            p.update(pi["player"])
+            except KeyError: pass
+
+        data.pop("participants"); data.pop("participantIdentities")
         return data
 
-    @property
-    def timeline(self) -> "Timeline":
-        return Timeline(id=self.id, platform=self.platform)
-
-
-class MatchTimeline(Match, PyotCore):
-    
-    class Meta(Match.Meta):
-        rules = {
-            "match_v4_tournament_match": ["tournament_code", "id"],
-            "match_v4_match": ["id"],
-            "match_v4_timeline": ["id"],
-        }
-
-    async def get(self, sid: str = None, pipeline: str = None, keep_raw: bool = False):
-        '''Awaitable. Get this object from the pipeline.\n
-        `sid` id identifying the session on the pipeline to reuse.\n
-        `pipeline` key identifying the pipeline to execute against.\n
-        `keep_raw` flag for storing raw data of the request as a dictionary.\n
-        '''
-        # pylint: disable=no-member
-        if pipeline:
-            self.set_pipeline(pipeline)
-        token1 = await self.create_token(search="match" if "tournament_code" not in self.__dict__ else "tournament")
-        token2 = await self.create_token(search="timeline")
-        task1 = asyncio.create_task(self._meta.pipeline.get(token1, sid))
-        task2 = asyncio.create_task(self._meta.pipeline.get(token2, sid))
-        data1 = await task1
-        data2 = await task2
-        if keep_raw:
-            self._meta.raw_data = {'match': fast_copy(data1), 'timeline': fast_copy(data2)}
-        self._meta.data = self._transform(data1, data2)
-        self._fill()
-        return self
-
-    def _transform(self, data1, data2):
-        data = super()._transform(data1)
-        teams = data["teams"]
+    def _inject_timeline(self, data):
+        teams = self._meta.data["teams"]
         frames = {}
         events = {}
-        for val in data2["frames"][0]["participantFrames"].values():
+        for val in data["frames"][0]["participantFrames"]:
             frames[val["participantId"]] = []
             events[val["participantId"]] = []
-        for frame in data2["frames"]:
-            for key, val in frame["participantFrames"].items():
-                p = val["participantId"]
-                frames[p].append(val)
+        for frame in data["frames"]:
+            for val in frame["participantFrames"]:
+                frames[val["participantId"]].append(val)
             for event in frame["events"]:
-                p = 0
-                if "participantId" in event:
-                    p = int(event["participantId"])
-                elif "creatorId" in event:
-                    p = int(event["creatorId"])
-                elif "killerId" in event:
-                    p = int(event["killerId"])
-                if p != 0:
-                    events[p].append(event)
+                try: events[event["participantId"]].append(event)
+                except KeyError:
+                    try: events[event["creatorId"]].append(event)
+                    except KeyError:
+                        try:
+                            events[event["killerId"]].append(event)
+                            events[event["victimId"]].append(event)
+                        except KeyError:
+                            pass
         for key in frames:
-            for i in range(len(teams)):
+            for team in teams:
                 found = False
-                for j in range(len(teams[i]["participants"])):
-                    if teams[i]["participants"][j]["participantId"] == key:
-                        teams[i]["participants"][j]["timeline"]["frames"] = frames[key]
-                        teams[i]["participants"][j]["timeline"]["events"] = events[key]
+                for participant in team["participants"]:
+                    if participant["participantId"] == key:
+                        participant["timeline"]["frames"] = frames[key]
+                        participant["timeline"]["events"] = events[key]
                         found = True
                         break
                 if found: break
         return data
 
+    @handle_import_error("roleml")
+    def roleml(self):
+        try:
+            roles = roleml.predict(self._meta.raw_data, self._meta.raw_timeline)
+        except AttributeError as e:
+            raise TypeError("This method requires timeline data to execute") from e
+        for team in self.teams:
+            for participant in team.participants:
+                participant.timeline.position = roles[participant.id]
+                participant.timeline._meta.data['position'] = roles[participant.id]
+        return roles
+
+    @handle_import_error("roleidentification")
+    def roleidentification(self):
+        resp = {}
+        for team in self.teams:
+            roles = dict_key_value_swap(roleidentification.get_roles(
+                champion_roles.get(),
+                [participant.champion_id for participant in team.participants]
+            ))
+            resp[team.id] = roles
+            for participant in team.participants:
+                participant.timeline.position = roles[participant.champion_id]
+                participant.timeline._meta.data['position'] = roles[participant.champion_id]
+        return resp
+
+    def raw_timeline(self):
+        return self._meta.raw_timeline
+
+    @property
+    def creation(self) -> datetime:
+        return datetime.fromtimestamp(self.creation_millis//1000)
+
+    @property
+    def duration(self) -> datetime:
+        return timedelta(seconds=self.duration_secs)
+
+    @property
+    def timeline(self) -> "Timeline":
+        return Timeline(id=self.id, platform=self.platform)
+
+    @property
+    def blue_team(self) -> MatchTeamData:
+        return next(team for team in self.teams if team.id == 100)
+
+    @property
+    def red_team(self) -> MatchTeamData:
+        return next(team for team in self.teams if team.id == 200)
+
+    @property
+    def participants(self) -> List[MatchParticipantData]:
+        participants = []
+        for team in self.teams:
+            participants += team.participants
+        return participants
+
 
 class Timeline(PyotCore):
     frames: List[MatchFrameMinuteData]
-    events: List[MatchEventMinuteData]
+    interval_millis: int
     interval: timedelta
 
     class Meta(PyotCore.Meta):
         rules = {"match_v4_timeline": ["id"]}
-
-    def __getattribute__(self, name):
-        if name == "interval":
-            return timedelta(milliseconds=super().__getattribute__(name))
-        return super().__getattribute__(name)
+        renamed = {"frame_interval": "interval_millis"}
 
     def __init__(self, id: int = None, platform: str = None):
         self._lazy_set(locals())
 
     def _transform(self, data):
-        new_data = {
-            "frames": [],
-            "events": [],
-            "interval": data["frameInterval"]
-        }
-        for f in data["frames"]:
-            new_data["frames"].append(list(f["participantFrames"].values()))
-            new_data["events"].append(f["events"])
-        return new_data
+        data = data.copy()
+        data["frames"] = data["frames"].copy()
+        for i, f in enumerate(data["frames"]):
+            data["frames"][i] = f.copy()
+            data["frames"][i]["participantFrames"] = list(f["participantFrames"].values())
+        return data
+
+    @property
+    def interval(self) -> datetime:
+        return timedelta(milliseconds=self.interval_millis)
 
 
 class MatchHistory(PyotCore):
@@ -613,7 +576,7 @@ class MatchHistory(PyotCore):
     end_index: int
     total_games: int
     account_id: str
-    
+
     class Meta(PyotCore.Meta):
         allow_query = True
         renamed = {"matches": "entries"}
@@ -635,7 +598,7 @@ class MatchHistory(PyotCore):
 
     def query(self, champion_ids: List[int] = None, queue_ids: List[int] = None, season_ids: List[int] = None, end_time: int = None, begin_time: int = None, end_index: int = None, begin_index: int = None):
         '''Add query parameters to the object.'''
-        kargs = {key if key[-3:] != "ids" else key[:-3] : val for (key,val) in locals().items()}
+        kargs = {key if key[-3:] != "ids" else key[:-3] : val for (key, val) in locals().items()}
         for i in [champion_ids, queue_ids, season_ids]:
             if not isinstance(i, list) and i is not None:
                 raise RuntimeError("Query parameters 'champion_ids', 'queue_ids' and 'season_ids' must be a list of values")
@@ -647,8 +610,8 @@ class MatchHistory(PyotCore):
         return [Match(id=entry.id, platform=entry.platform) for entry in self.entries]
 
     @property
-    def match_timelines(self) -> List[MatchTimeline]:
-        return [MatchTimeline(id=entry.id, platform=entry.platform) for entry in self.entries]
+    def match_timelines(self) -> List[Match]:
+        return [Match(id=entry.id, include_timeline=True, platform=entry.platform) for entry in self.entries]
 
     @property
     def timelines(self) -> List[Timeline]:
@@ -667,7 +630,7 @@ class Matches(PyotCore):
     class Meta(PyotCore.Meta):
         rules = {"match_v4_tournament_matches": ["tournament_code"]}
         raws = ["ids"]
-    
+
     def __getitem__(self, item):
         if not isinstance(item, int):
             return super().__getitem__(item)
@@ -685,10 +648,10 @@ class Matches(PyotCore):
 
     @property
     def match_timelines(self) -> List[Match]:
-        return [MatchTimeline(id=id_, tournament_code=self.tournament_code, platform=self.platform) for id_ in self.ids]
+        return [Match(id=id_, tournament_code=self.tournament_code, include_timeline=True, platform=self.platform) for id_ in self.ids]
 
     @property
-    def timelines(self) -> List[Match]:
+    def timelines(self) -> List[Timeline]:
         return [Timeline(id=id_, platform=self.platform) for id_ in self.ids]
 
     def _transform(self, data):
