@@ -1,6 +1,8 @@
 from datetime import datetime, timedelta
 import pickle
 
+from .common import thread_run
+
 
 class PtrCache:
     '''
@@ -16,12 +18,12 @@ class PtrCache:
         self.expiration = expiration
         self.max_entries = max_entries
 
-    def get(self, name: str, func = None):
+    def get(self, name: str, func=None, lazy: bool = False):
         '''
         Get an object from the cache.
 
-        `func` will be called when provided and if object doesn't exist,
-        put the returned value to the cache and return it.
+        `func` will be called when provided and if object doesn't exist, set the returned value before returning.\n
+        `lazy` flag if `func` needs to be called before running it, therefore achieve lazy eval during runtime.  
         '''
         try:
             data = self.objects[name]
@@ -32,16 +34,17 @@ class PtrCache:
         except KeyError:
             if func is None:
                 return None
-        response = func()
+        response = func()() if lazy else func()
         self.set(name, response)
         return response
 
-    async def aget(self, name: str, coro = None):
+    async def aget(self, name: str, coro=None, lazy: bool = False):
         '''
         Async get an object from the cache.
 
-        `coro` will be awaited when provided and if object doesn't exist,
-        put the returned value to the cache and return it.
+        `coro` will be awaited when provided and if object doesn't exist, set the returned value before returning.\n
+        `lazy` flag if `coro` needs to be called before running it, therefore achieve lazy eval during runtime.\n
+
         If the `coro` doesn't need to be awaited it will be closed and not raise warnings.
         '''
         try:
@@ -49,13 +52,13 @@ class PtrCache:
             if data[1] is not None and data[1] < datetime.now():
                 del self.objects[name]
                 raise KeyError(name)
-            if coro:
+            if not lazy and coro:
                 coro.close()
             return data[0]
         except KeyError:
             if coro is None:
                 return None
-        response = await coro
+        response = (await coro()) if lazy else (await coro)
         self.set(name, response)
         return response
 
@@ -82,7 +85,7 @@ class PtrCache:
         self.objects = dict()
 
 
-def ptr_cache(expiration=60*60*3, max_entries=2000):
+def ptr_cache(expiration=60*60*3, max_entries=5000):
     '''Create an PtrCache and return it.'''
     return PtrCache(expiration, max_entries)
 
@@ -103,3 +106,29 @@ class FrozenGenerator:
 def frozen_generator(li):
     '''Create a FrozenGenerator and return it.'''
     return FrozenGenerator(li)
+
+
+class AutoData:
+
+    def __init__(self, func, interval=60*60*3):
+        self.interval = interval
+        self.func = func
+        self.data = None
+        self.next_update = datetime.now()
+        self.updating = False
+
+    def get(self):
+        if self.next_update < datetime.now() and not self.updating:
+            self.updating = True
+            self.data = self.func()
+            self.updating = False
+            self.next_update = datetime.now() + timedelta(seconds=self.interval)
+        return self.data
+
+    async def aget(self):
+        if self.next_update < datetime.now() and not self.updating:
+            self.updating = True
+            self.data = await thread_run(self.func)
+            self.updating = False
+            self.next_update = datetime.now() + timedelta(seconds=self.interval)
+        return self.data
